/*
 * isr.c
 *
 * Created: 01.05.2015 20:54:29
 *  Author: Norbert
 */ 


#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "isr.h"
#include "main.h"
#include "dcf_77.h"

//////////////////////////////////////////////////////////////////////////
// feste Variablem im SRAM, gloabal, nach auen sichtbar
volatile uint8_t Tasten;	//Variable fr entprellte Taste 
time_t _time;				//_time ist die Variable vom Type time_t !

//////////////////////////////////////////////////////////////////////////
// Konstanten im flash, Anzahl der Tage im Jahr, Mit & Ohne Schaltjahr 
static const uint8_t daytab1[] PROGMEM = {31,28,31,30,31,30,31,31,30,31,30,31};
static const uint8_t daytab2[] PROGMEM = {31,29,31,30,31,30,31,31,30,31,30,31};
//////////////////////////////////////////////////////////////////////////
// Soft PWM
// 16MHz / 256 = 62kHz /80 Helligkeits-Stufen = 775Hz / 6 Spalten = 130 Hz !! 
ISR(TIMER0_OVF_vect)
{
	static uint8_t spalte=0x01;
	static uint8_t pwm_counter;
	
	//PORTD|=(spalte);			//aktuelle Spalte lschen = HIGH
	
	if(++pwm_counter==PWM_count_max)
	{
		PORTD|=(spalte);			//aktuelle Spalte lschen = HIGH
		pwm_counter=0;
		spalte<<=1;
		if(spalte==0x40) spalte=0x01;
		switch (spalte){
			case 0x01: PORTB=_led.digit[0];break;
			case 0x02: PORTB=_led.digit[1];break;
			case 0x04: PORTB=_led.digit[2];break;
			case 0x08: PORTB=_led.digit[3];break;
			case 0x10: PORTB=_led.digit[4];break;
			case 0x20: PORTB=_led.digit[5];break;			
		}
		PORTD&=~(spalte);				//neue Spalte einschalten = LOW	
	}
	
	// Soft PWM wert schon erreicht ? dann LED Spalte aus (HIGH) else LED Spalte an (LOW)
	if(pwm_counter>=_led.soft_pwm_wert) PORTD|=(spalte); else PORTD&=~(spalte);	
}

static void tasten_auswerten(void)
{
	static uint8_t old_key;
	uint8_t key=(~Tasten_in & key_mask);
	if (key!=old_key)
	{
		if (key) Tasten=key;
		old_key=key;
	}
}


//////////////////////////////////////////////////////////////////////////
// Timer 1, 16bit, CTC Modus, alle 10ms
// Zeitbasis wird hier gemacht, "_time" ist eine Struktur (ist bersichtlicher)
ISR(TIMER1_COMPA_vect)
{
	uint8_t mflag=0;
	if (++_time.ms==100)			
	{
		_time.ms=0;						
		if (timeout) timeout--;
		if (++_time.sekunden>=60)
		{
			_time.sekunden=0;
			if (++_time.minuten>=60)
			{
				_time.minuten=0;
				if(_DCF.SYNCRO>50) _DCF.SYNCRO++;		//Wenn DCF_zeit OK, erhhe jede Stunde um 1 !!
				if(++_time.stunden>=24)
				{
					_time.stunden=0;
					if (++_time.wochentag>=8) _time.wochentag=1;
					++_time.tag;					// Tag +1
					
					//////////////////////////////////////////////////////////////////////////
					// Wenn aktuelles Jahr/4 und Rest>0 , dann kein Schaltjahr(365 Tage im Jahr)
					//	else Jahr/4 und Rest 0, dann Schaltjahr (366 Tage)
					if (_time.jahr%4){if (_time.tag>pgm_read_byte(&daytab1[_time.monat-1])) mflag=1;}	//365 Tage
					else{if (_time.tag>pgm_read_byte(&daytab2[_time.monat-1])) mflag=1;}				//366 Tage
					
					//////////////////////////////////////////////////////////////////////////
					// Monatsflag gestzt ???
					if (mflag){
						_time.tag=1;
						if (++_time.monat>=13){_time.monat=1,_time.jahr++;}}	
				}
			}
		}
	}
	
	tasten_auswerten();
	if (_DCF.SYNCRO<50) DCF_erfassen();
}