/*
 * main.c
 *
 * Created: 01.05.2015 20:42:21
 *  Author: Norbert
 *
 *************************************************
 *	Neuauflage der bekannten Binr Uhr von mir ! *
 *************************************************
 *	kompatibel mit Atmega48/88/168/328
 *	oder Arduino NANO/UNO
 * 
 * mit DCF Empfnger (von ELV z.Bsp.)
 * kann mit Atmega48/88/168/328 nachgebaut werden, sind alle gleich vom Aufbau
 * haben nur Unterschiedlich viel Flash-Speicher (4k-, 8k-, 16k-, 32k-Byte)
 * Mit 16MHz externen Quarz zu betreiben ! wegen genauer Uhrzeit !
 * Fuse fr Atmega48/88/168/328 - LOW:FF  HIGH:DF
 *
 *
 ******************************************************************************
 * Optimazion Level:
 * Os = kleinste Gre des erzeugenten Codes
 * 		1686 Bytes Program / 115 Bytes Data
 *****************************************************************************
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include "main.h"				// fr F_CPU, Tasten_def., LED_def.
#include "isr.h"				// ISR einbinden, + Soft PWM
#include "dcf_77.h"				// def. fr DCF Modul ! + Pin Belegung
#include <util/delay.h>


//////////////////////////////////////////////////////////////////////////
// Globale Variable (static = nur hier sichtbar, volatile = nach auen sichtbar) 
static uint8_t Anzeige_modus=0;		//statische Variable zum steuern der Anzeige
volatile uint8_t timeout=0;			//nach auen sichtbar !, Zeit bis wieder auf Zeit_ausgeben geweckselt wird !

//////////////////////////////////////////////////////////////////////////
// eine Aufzhlung, Zeit_ausgeben=0, Datum_ausgeben=1, usw.
enum {	Zeit_ausgeben,			//aktuelle Zeit ausgeben
		Datum_ausgeben,			//aktuelles Datum ausgeben
		Jahr_ausgeben};			//aktuelles Jahr ausgeben

		
// ein Beispiel vorladen zum Testen der Anzeige !
inline static void vorgaben_laden(void)
{						// heute ist
	_time.wochentag=1;	//	Montag
	_time.tag=1;		// der 01.
	_time.monat=1;		// 01.
}

inline static void DCF_LED_AUSGEBEN(void)
{
	if (~DCF_IN & DCF_PIN)  DCF_LED_Port&=~DCF_LED_PIN;		//DCF_LED ON
	_delay_ms(0.5);
	DCF_LED_Port|=DCF_LED_PIN;								//DCF_LED OFF
	_delay_ms(9.5);
	
}


//////////////////////////////////////////////////////////////////////////
// Hauptprogramm
int main(void)
{		
	DDRB=0xFF;	PORTB=0x00;			//PortB Ausgang, alles LOW
	DDRD=0xFF;	PORTD=0xFF;			//PortB Ausgang, alles LOW
	DDRC&=~key_mask;				//Port = Eingang (Taster_0 + Taster_1)
	PORTC|=key_mask;				// PullUp ON (Taster_0 + Taster_1)
	DCF_init();						//Port fr DCF konfigurieren !
	
	//////////////////////////////////////////////////////////////////////////
	// Timer_0, 8bit, fr soft PWM
	TCCR0B=(1<<CS00);					//Prescaler 1, voller Sytemtakt (16MHz)
	TIMSK0|=(1<<TOIE0);					//Timer Overflow Flag setzen 
	_led.soft_pwm_wert=PWM_startwert;	// Werte fr soft-PWM setzen !
	
	//////////////////////////////////////////////////////////////////////////
	// Timer_1, 16bit, im CTC MODUS, fr Zeitgrundlage !
	OCR1A=(F_CPU/8/100)-1;				//Compare Match Wert A , vorladen (16MHz/8/100-1=19999)
	TCCR1B=(1<<CS11);					//Prescaler 8, bei 16MHz
	TCCR1B|=(1<<WGM12);					//CTC Mode (Mode 4 laut Datenblatt)
	TIMSK1|=(1<<OCIE1A);				//Compare Match A von Timer 1
	sei();								//interrupts freigeben
	
	
	//////////////////////////////////////////////////////////////////////////
	vorgaben_laden();					// zum testen mal was vorladen !
	//////////////////////////////////////////////////////////////////////////
	
    while(1)
    {
		// verzweige je nach Anzeige_mod !
		switch (Anzeige_modus){
			case Zeit_ausgeben:				//Zeit ausgeben !
				_led.digit[0]=_time.sekunden%10;
				_led.digit[1]=_time.sekunden/10;
				_led.digit[2]=_time.minuten%10;
				_led.digit[3]=_time.minuten/10;
				_led.digit[4]=_time.stunden%10;
				_led.digit[5]=_time.stunden/10;
				break;
				
			case Datum_ausgeben:			// Datum ausgeben
				_led.digit[0]=_time.monat%10;
				_led.digit[1]=_time.monat/10;
				_led.digit[2]=_time.tag%10;
				_led.digit[3]=_time.tag/10;
				_led.digit[4]=_time.wochentag;
				_led.digit[5]=0;
				break;
				
			case Jahr_ausgeben:				// Jahr ausgeben 
				_led.digit[0]=_time.jahr%10;
				_led.digit[1]=0;
				_led.digit[2]=_time.jahr/10;
				_led.digit[3]=0;
				_led.digit[4]=2;
				_led.digit[5]=0;
				break;
			
		}
		
		//////////////////////////////////////////////////////////////////////////
		// DCF_LED ansteuern (wenn Zeit noch nicht syncron) inkl. einer verzgerung von 10ms !
		if (_DCF.SYNCRO<50) DCF_LED_AUSGEBEN();
		
		//////////////////////////////////////////////////////////////////////////
		//DCF_vollstndig = dann berprfe das Paket ! Rckgabewert=eventuelle Fehler !
		if(_DCF.SYNCRO==DCF_PAKET_da) DCF_auswerten();
		// DCF_paket in Ordnung
		if(_DCF.SYNCRO==DCF_PAKET_IO) DCF_zeit_uebernehmen();
		
		//////////////////////////////////////////////////////////////////////////
		// Wurde Taste gedrckt ??
		switch (Tasten){
			case Taste_0:
				Tasten=0;
				_led.soft_pwm_wert<<=1;		//*2
				if(_led.soft_pwm_wert>PWM_count_max-20) _led.soft_pwm_wert=PWM_startwert;
				break;
				
			case Taste_1:
				Tasten=0;
				//Anzeige_modus++;
				if (++Anzeige_modus>Jahr_ausgeben) Anzeige_modus=Zeit_ausgeben;
				timeout=Delay_15s;
				break;
			
		}// end of switch(Tasten)
		
		//////////////////////////////////////////////////////////////////////////
		// Wenn Anzeige_mod!=Zeit anzeigen && timeout==0, dann springe wieder zu Zeit anzeigen !
		if ((timeout==0) && (Anzeige_modus!=Zeit_ausgeben)) Anzeige_modus=Zeit_ausgeben;
		
    }//end of while(1)
		
}// end of int main(void)